# Copyright (c) 2015, Xhive
# See license.txt

import unittest

import xhiveframework

from education.education.doctype.course_activity.test_course_activity import \
    make_course_activity
from education.education.doctype.program.test_program import setup_program
from education.education.doctype.student.test_student import (create_student,
                                                              get_student)


class TestCourseEnrollment(unittest.TestCase):
	def setUp(self):
		setup_program()
		student = create_student(
			{
				"first_name": "_Test First",
				"last_name": "_Test Last",
				"email": "_test_student_1@example.com",
			}
		)
		program_enrollment = student.enroll_in_program("_Test Program")
		course_enrollment = xhiveframework.db.get_value(
			"Course Enrollment",
			{
				"course": "_Test Course 1",
				"student": student.name,
				"program_enrollment": program_enrollment.name,
			},
			"name",
		)
		make_course_activity(course_enrollment, "Article", "_Test Article 1-1")

	def test_get_progress(self):
		student = get_student("_test_student_1@example.com")
		program_enrollment_name = xhiveframework.get_list(
			"Program Enrollment", filters={"student": student.name, "Program": "_Test Program"}
		)[0].name
		course_enrollment_name = xhiveframework.get_list(
			"Course Enrollment",
			filters={
				"student": student.name,
				"course": "_Test Course 1",
				"program_enrollment": program_enrollment_name,
			},
		)[0].name
		course_enrollment = xhiveframework.get_doc("Course Enrollment", course_enrollment_name)
		progress = course_enrollment.get_progress(student)
		finished = {
			"content": "_Test Article 1-1",
			"content_type": "Article",
			"is_complete": True,
		}
		self.assertTrue(finished in progress)
		xhiveframework.db.rollback()

	def tearDown(self):
		for entry in xhiveframework.db.get_all("Course Enrollment"):
			xhiveframework.delete_doc("Course Enrollment", entry.name)

		for entry in xhiveframework.db.get_all("Program Enrollment"):
			doc = xhiveframework.get_doc("Program Enrollment", entry.name)
			doc.cancel()
			doc.delete()
