# Copyright (c) 2015, Xhive
# For license information, please see license.txt


import xhiveframework
from xhiveframework import _
from xhiveframework.model.document import Document
from xhiveframework.utils.csvutils import getlink


class Guardian(Document):
	def __setup__(self):
		self.onload()

	def onload(self):
		"""Load Students for quick view"""
		self.load_students()

	def load_students(self):
		"""Load `students` from the database"""
		self.students = []
		students = xhiveframework.get_all(
			"Student Guardian", filters={"guardian": self.name, "parenttype":"Student"}, fields=["parent"]
		)
		for student in students:
			self.append(
				"students",
				{
					"student": student.parent,
					"student_name": xhiveframework.db.get_value("Student", student.parent, "student_name"),
				},
			)

	def validate(self):
		self.students = []


@xhiveframework.whitelist()
def invite_guardian(guardian):
	guardian_doc = xhiveframework.get_doc("Guardian", guardian)
	if not guardian_doc.email_address:
		xhiveframework.throw(_("Please set Email Address"))
	else:
		guardian_as_user = xhiveframework.get_value("User", dict(email=guardian_doc.email_address))
		if guardian_as_user:
			xhiveframework.msgprint(
				_("User {0} already exists").format(getlink("User", guardian_as_user))
			)
			return guardian_as_user
		else:
			user = xhiveframework.get_doc(
				{
					"doctype": "User",
					"first_name": guardian_doc.guardian_name,
					"email": guardian_doc.email_address,
					"user_type": "Website User",
					"send_welcome_email": 1,
				}
			).insert(ignore_permissions=True)
			xhiveframework.msgprint(_("User {0} created").format(getlink("User", user.name)))
			return user.name
