# Copyright (c) 2015, Xhive
# License: GNU General Public License v3. See license.txt


import xhiveframework
from xhiveerp.setup.doctype.holiday_list.holiday_list import is_holiday
from xhiveframework import _, msgprint
from xhiveframework.utils import formatdate

from education.education.doctype.student_attendance.student_attendance import (
	get_holiday_list,
)


def execute(filters=None):
	if not filters:
		filters = {}

	if not filters.get("date"):
		msgprint(_("Please select date"), raise_exception=1)

	columns = get_columns(filters)
	date = filters.get("date")

	holiday_list = get_holiday_list()
	if is_holiday(holiday_list, filters.get("date")):
		msgprint(
			_("No attendance has been marked for {0} as it is a Holiday").format(
				xhiveframework.bold(formatdate(filters.get("date")))
			)
		)

	absent_students = get_absent_students(date)
	leave_applicants = get_leave_applications(date)
	if absent_students:
		student_list = [d["student"] for d in absent_students]

	data = []
	for student in absent_students:
		if not student.student in leave_applicants:
			row = [student.student, student.student_name, student.student_group]
			stud_details = xhiveframework.db.get_value(
				"Student",
				student.student,
				["student_email_id", "student_mobile_number"],
				as_dict=True,
			)

			if stud_details.student_email_id:
				row += [stud_details.student_email_id]
			else:
				row += [""]

			if stud_details.student_mobile_number:
				row += [stud_details.student_mobile_number]
			else:
				row += [""]

			data.append(row)

	return columns, data


def get_columns(filters):
	columns = [
		_("Student") + ":Link/Student:90",
		_("Student Name") + "::150",
		_("Student Group") + "::180",
		_("Student Email Address") + "::180",
		_("Student Mobile No.") + "::150",
	]
	return columns


def get_absent_students(date):
	absent_students = xhiveframework.db.sql(
		"""
		SELECT student, student_name, student_group
		FROM `tabStudent Attendance`
		WHERE
			status='Absent' and docstatus=1 and date = %s
		ORDER BY
			student_group, student_name""",
		date,
		as_dict=1,
	)
	return absent_students


def get_leave_applications(date):
	leave_applicants = []
	leave_applications = xhiveframework.db.sql(
		"""
		SELECT student
		FROM
			`tabStudent Leave Application`
		WHERE
			docstatus = 1 and mark_as_present = 1 and
			from_date <= %s and to_date >= %s
	""",
		(date, date),
	)
	for student in leave_applications:
		leave_applicants.append(student[0])

	return leave_applicants


def get_transportation_details(date, student_list):
	academic_year = xhiveframework.get_all(
		"Academic Year",
		filters=[["year_start_date", "<=", date], ["year_end_date", ">=", date]],
	)
	if academic_year:
		academic_year = academic_year[0].name
	elif xhiveframework.defaults.get_defaults().academic_year:
		academic_year = xhiveframework.defaults.get_defaults().academic_year
	else:
		return {}

	transportation_details = xhiveframework.get_all(
		"Program Enrollment",
		fields=["student", "mode_of_transportation", "vehicle_no"],
		filters={
			"student": ("in", student_list),
			"academic_year": academic_year,
			"docstatus": ("not in", ["2"]),
		},
	)
	transportation_map = {}
	for d in transportation_details:
		transportation_map[d.student] = [d.mode_of_transportation, d.vehicle_no]
	return transportation_map
